*  MON6303.ASM - 6303 (or 6803) Debug monitor for use with NOICE03
*
*  Copyright (c) 1993-1997 by John Hartman
*
*  Modification History:
*       20-Jun-97 JLH ported from 6805 and HC11 versions
*	02-Apr-98 JLH assemble with either Motorola or Dunfield
*
*============================================================================
*
*  To customize for a given target, you must change code in the
*  hardware equates, the string TSTG, and the routines RESET and REWDT.
*  You may or may not need to change GETCHAR, PUTCHAR, depending on
*  how peculiar your UART is.
*
*  This file has been assembled with the Motorola Freeware assembler
*  available from the Motorola Freeware BBS and elsewhere.
*
*  This file may also be assembled with the Dunfield assembler
*
*  To add mapped memory support:
*       1) Define map port MAPREG here
*       2) Define or import map port RAM image MAPIMG here if MAPREG is
*          write only.  (The application code must update MAPIMG before
*          outputing to MAPREG)
*       3) Search for and modify MAPREG, MAPIMG, and REG_PAGE usage below
*       4) In TSTG below edit "LOW AND HIGH LIMIT OF MAPPED MEM"
*          to appropriate range (typically 4000H to 07FFFH for two-bit MMU)
*
*============================================================================
*
*  Hardware definitions
*
STACK_TOP       EQU     $1F00           TOP OF STACK (BELOW MONITOR RAM)
RAM_START       EQU     $1F00           START OF MONITOR RAM (OK IF NOT PAGE 0)
RAM_VECT        EQU     $1FEE           START OF USER VECTORS (IN RAM)
NVEC            EQU     9		NUMBER OF INTERRUPT VECTORS
*
ROM_START       EQU     $FC00           START OF MONITOR CODE
HARD_VECT       EQU     $FFEE           START OF HARDWARE VECTORS

        PAGE
*============================================================================
*
*
*============================================================================
*  HARDWARE PLATFORM CUSTOMIZATIONS
*============================================================================
*
*  Put you UART equates here
*  (These are for a DUART)
DUART           EQU     $17f0
SER_STATUS      EQU     $17f1
SER_RXDATA      EQU     $17f3
SER_TXDATA      EQU     $17f3
RXRDY           EQU     $01
TXRDY           EQU     $04
*
*============================================================================
*
*  RAM definitions
        ORG     RAM_START
*
*  Save vale of SP, as discovered at entry
TASK_REGS
REG_STATE       RMB     1
REG_PAGE        RMB     1
REG_SP		RMB	2
REG_X           RMB	2
REG_B           RMB	1
REG_A           RMB	1
REG_CC          RMB	1
REG_PC          RMB	2
*
*  Declare size of register block
TASK_REG_SZ     EQU     *-TASK_REGS
*
*  Address pointers
MEMPTR		RMB	2
BUFPTR		RMB	2
TEMPWORD	RMB	2
*
*  Communications buffer
*  (Must be at least as long as the longer of TASK_REG_SZ or TSTG_SIZE.
*  At least 19 bytes recommended.  Larger values may improve speed of NoICE
*  download and memory move commands.)
COMBUF_SIZE     EQU     128             DATA SIZE FOR COMM BUFFER
COMBUF          RMB     2+COMBUF_SIZE+1 BUFFER ALSO HAS FN, LEN, AND CHECK
*
RAM_END         EQU     *               ADDRESS OF TOP+1 OF RAM
*
*===========================================================================
        ORG     ROM_START
*
*  Power on reset
RESET
*
*  Set CPU mode to safe state
        SEI                             INTERRUPTS OFF (WE MAY JUMP HERE)
*
*  Initialize 6303/6803 hardware
*
        LDS     #STACK_TOP              CLEAN STACK IS HAPPY STACK
*
*  Initialize your UART here (This is for the 2681 DUART)
        LDAA    #$13            MODE1: NO PARITY, 8 DATA
        STAA    DUART

        LDAA    #$CC            CLOCK SELECT: 19200 BAUD RX AND TX
        STAA    DUART+1

        LDAA    #$07            MODE2: ONE STOP BIT
        STAA    DUART

        LDAA    #$80            ACR: COUNTER MODE (ALLOWS 19200 BAUD)
        STAA    DUART+4

        LDAA    #$00            INT MASK: NONE
        STAA    DUART+5

        LDAA    #$08            OUTPUT CONFIG: TxCB
        STAA    DUART+$D

        LDAA    #$15            CMD REGISTER:  ENABLE TX, RX
        STAA    DUART+2
*
*----------------------------------------------------------------------------
*
*  Initialize RAM interrupt vectors
        LDX     #RAM_VECT       POINTER TO RAM VECTORS
        LDAB    #NVEC/2         NUMBER OF VECTORS
        STAB	TEMPWORD
RST10   LDD     #INT_ENTRY      ADDRESS OF DEFAULT HANDLER
	STD     0,X             SET VECTOR
        INX
        INX
        DEC	TEMPWORD
        BNE     RST10
*
*  Initialize memory paging variables and hardware (if any)
        LDAA    #0
        STAA    REG_PAGE                NO PAGE YET
*;*     STAA    MAPIMG
*;*     STAA    MAPREG                  set hardware map
*
*  Initialize user registers
        LDD     #STACK_TOP
        STAA    REG_SP+1                INIT USER'S STACK POINTER MSB
        STAB    REG_SP                  LSB
        LDD     #0
        STD     REG_PC
        STAA    REG_A
        STAA    REG_B
        STD     REG_X
        STAA    REG_STATE               initial state if "RESET"
*
*  Initialize non-zero registers
        LDAA    #$50                    disable interrupts in user program
        STAA    REG_CC
*
*  Set function code for "GO".  Then if we are here because of a reset
*  after being told to GO, we will come back with registers so user can 
*  see the reset
        LDAA    #FN_RUN_TARG
        STAA    COMBUF
        JMP     RETURN_REGS             DUMP REGS, ENTER MONITOR
*
*===========================================================================
*  Get a character to A
*
*  Return A=char, CY=0 if data received
*         CY=1 if timeout (0.5 seconds)
*
*  Uses 6 bytes of stack including return address
*
GETCHAR PSHX
        LDX     #0              LONG TIMEOUT
GC10    JSR     REWDT           PREVENT WATCHDOG TIMEOUT
        DEX
*;*     BEQ     GC90            EXIT IF TIMEOUT
*(Disable timeout in most cases...)
        LDAA    SER_STATUS      READ DEVICE STATUS
        ANDA    #RXRDY
        BEQ     GC10            NOT READY YET.
*
*  Data received:  return CY=0. data in A
        CLC                     CY=0
        LDAA    SER_RXDATA      READ DATA
        PULX
        RTS
*
*  Timeout:  return CY=1
GC90    SEC                     CY=1
        PULX
        RTS
*
*===========================================================================
*  Output character in A
*
*  Uses 5 bytes of stack including return address
*
PUTCHAR PSHA
PC10    JSR     REWDT           PREVENT WATCHDOG TIMEOUT
        LDAA    SER_STATUS      CHECK TX STATUS
        ANDA    #TXRDY          TX READY ?
        BEQ     PC10
        PULA
        STAA    SER_TXDATA      TRANSMIT CHAR.
        RTS
*
*======================================================================
*  Reset watchdog timer.  Must be called at least once every little while
*  or COP interrupt will occur
*
*  Uses 2 bytes of stack including return address
*
REWDT   RTS
*
*===========================================================================
*  Read byte from MEMPTR
READMEM	LDX	MEMPTR
	LDAA	0,X
        RTS

*===========================================================================
*  Write byte to MEMPTR
WRITEMEM
	LDX	MEMPTR
	STAA	0,X
        RTS

*===========================================================================
*  Increment MEMPTR
INCMEM	LDX	MEMPTR
	INX
        STX	MEMPTR
        RTS

*===========================================================================
*  Read byte from BUFPTR
READBUF	LDX	BUFPTR
	LDAA	0,X
        RTS

*===========================================================================
*  Write byte to BUFPTR
WRITEBUF
	LDX	BUFPTR
	STAA	0,X
        RTS

*===========================================================================
*  Increment BUFPTR
INCBUF	LDX	BUFPTR
	INX
        STX	BUFPTR
        RTS

*======================================================================
*  Response string for GET TARGET STATUS request
*  Reply describes target:
TSTG    FCB     11                       2: PROCESSOR TYPE = 6303
        FCB     COMBUF_SIZE             3: SIZE OF COMMUNICATIONS BUFFER
        FCB     0                       4: NO TASKING SUPPORT
        FCB     0,0                     5,6: BOTTOM OF MAPPED MEM (LSB FIRST)
        FCB     0,0                     7,8: TOP OF MAPPED MEM (LSB FIRST)
        FCB     B1-B0                   9 BREAKPOINT INSTR LENGTH
B0      SWI                             10+ BREKAPOINT INSTRUCTION
B1      FCC     '6303/6803 monitor V1.0' DESCRIPTION, ZERO
        FCB     0
TSTG_SIZE       EQU     *-TSTG          SIZE OF STRING
*
*======================================================================
*  HARDWARE PLATFORM INDEPENDENT EQUATES AND CODE
*
*  Communications function codes.
FN_GET_STAT     EQU     $FF    reply with device info
FN_READ_MEM     EQU     $FE    reply with data
FN_WRITE_M      EQU     $FD    reply with status (+/-)
FN_READ_RG      EQU     $FC    reply with registers
FN_WRITE_RG     EQU     $FB    reply with status
FN_RUN_TARG     EQU     $FA    reply (delayed) with registers
FN_SET_BYTE     EQU     $F9    reply with data (truncate if error)
FN_IN           EQU     $F8    input from port
FN_OUT          EQU     $F7    output to port
*
FN_MIN          EQU     $F7    MINIMUM RECOGNIZED FUNCTION CODE
FN_ERROR        EQU     $F0    error reply to unknown op-code
*
*===========================================================================
*  Common handler for default interrupt handlers
*  Enter with A=interrupt code = processor state
*  All registers stacked, PC=next instruction
*
INT_ENTRY
        STAA    REG_STATE       SAVE STATE
*
*  Save registers from stack to reg block for return to master
*  Host wants least significant bytes first, so flip as necessary
        PULA
        STAA    REG_CC          CONDITION CODES
        PULA
        STAA    REG_B
        PULA
        STAA    REG_A
        PULA
        STAA    REG_X+1         MSB
        PULA
        STAA    REG_X           LSB
*
*  If this is a breakpoint (state = 1), then back up PC to point at SWI
*  (If another instruction is used for breakpoint, then DEX multiple 
*  times to match B1-B0 in TSTG)
        PULX                    PC AFTER INTERRUPT
        LDAA    REG_STATE
        CMPA    #1
        BNE     NOTBP           BR IF NOT A BREAKPOINT
        DEX                     ELSE BACK UP TO POINT AT SWI LOCATION
NOTBP   PSHX                    TRANSFER PC TO D
	PULA
        PULB
        STAA    REG_PC+1        MSB
        STAB    REG_PC          LSB
        TSX                     USER STACK POINTER PLUS 1
        DEX                     MAKE IT JUST LIKE THE REAL SP
        PSHX
	PULA
        PULB
        STAB    REG_SP          SAVE USER'S STACK POINTER (LSB)
        STAA    REG_SP+1        MSB
*
*  Save memory page
*;*     LDAA    MAPIMG          GET CURRENT USER MAP
        LDAA    #0              ... OR ZERO IF UNMAPPED TARGET
        STAA    REG_PAGE        SAVE USER'S PAGE
*
*  Return registers to master
        JMP     RETURN_REGS
*
*===========================================================================
*  Main loop:  wait for command frame from master
*
*  Uses 2 bytes of stack before jump to handlers
*
MAIN    LDS	REG_SP			CLEAN STACK IS HAPPY STACK
        LDX     #COMBUF                 BUILD MESSAGE HERE
*
*  First byte is a function code
        JSR     GETCHAR                 GET A FUNCTION
        BCS     MAIN                    JIF TIMEOUT: RESYNC
        CMPA    #FN_MIN
        BLO     MAIN                    JIF BELOW MIN: ILLEGAL FUNCTION
        STAA    0,X 	               SAVE FUNCTION CODE
        INX
*
*  Second byte is data byte count (may be zero)
        JSR     GETCHAR                 GET A LENGTH BYTE
        BCS     MAIN                    JIF TIMEOUT: RESYNC
        CMPA    #COMBUF_SIZE
        BHI     MAIN                    JIF TOO LONG: ILLEGAL LENGTH
        STAA    0,X                     SAVE LENGTH
        INX
        CMPA    #0
        BEQ     MA80                    SKIP DATA LOOP IF LENGTH = 0
*
*  Loop for data
        TAB                             SAVE LENGTH FOR LOOP
MA10    JSR     GETCHAR                 GET A DATA BYTE
        BCS     MAIN                    JIF TIMEOUT: RESYNC
        STAA    0,X                     SAVE DATA BYTE
        INX
        DECB
        BNE     MA10
*
*  Get the checksum
MA80    JSR     GETCHAR                 GET THE CHECKSUM
        BCS     MAIN                    JIF TIMEOUT: RESYNC
        PSHA                            SAVE CHECKSUM
*
*  Compare received checksum to that calculated on received buffer
*  (Sum should be 0)
        JSR     CHECKSUM
        PULB
        ABA
        BNE     MAIN                    JIF BAD CHECKSUM
*
*  Process the message.
        LDX     #COMBUF
        LDAA    0,X                     GET THE FUNCTION CODE
        LDAB    1,X                     GET THE LENGTH
        INX
        INX                             X POINTS AT DATA
        CMPA    #FN_GET_STAT
        BEQ     TARGET_STAT
        CMPA    #FN_READ_MEM
        BEQ     JREAD_MEM
        CMPA    #FN_WRITE_M
        BEQ     JWRITE_MEM
        CMPA    #FN_READ_RG
        BEQ     JREAD_REGS
        CMPA    #FN_WRITE_RG
        BEQ     JWRITE_REGS
        CMPA    #FN_RUN_TARG
        BEQ     JRUN_TARGET
        CMPA    #FN_SET_BYTE
        BEQ     JSET_BYTES
        CMPA    #FN_IN
        BEQ     JIN_PORT
        CMPA    #FN_OUT
        BEQ     JOUT_PORT
*
*  Error: unknown function.  Complain
        LDAA    #FN_ERROR
        STAA    COMBUF          SET FUNCTION AS "ERROR"
        LDAA    #1
        JMP     SEND_STATUS     VALUE IS "ERROR"
*
*  long jumps to handlers
JREAD_MEM       JMP     READ_MEM
JWRITE_MEM      JMP     WRITE_MEM
JREAD_REGS      JMP     READ_REGS
JWRITE_REGS     JMP     WRITE_REGS
JRUN_TARGET     JMP     RUN_TARGET
JSET_BYTES      JMP     SET_BYTES
JIN_PORT        JMP     IN_PORT
JOUT_PORT       JMP     OUT_PORT

*===========================================================================
*
*  Target Status:  FN, len
*
TARGET_STAT
        LDAB    #TSTG_SIZE              LENGTH OF REPLY
        STAB    COMBUF+1                SET SIZE IN REPLY BUFFER
        STX	BUFPTR
        LDX	#TSTG
        STX	MEMPTR
TS10    JSR	READMEM                 MOVE REPLY DATA TO BUFFER
	JSR	INCMEM
	JSR	WRITEBUF
        JSR	INCBUF
        DECB
        BNE     TS10
*
*  Compute checksum on buffer, and send to master, then return
        JMP     SEND

*===========================================================================
*
*  Read Memory:  FN, len, page, Alo, Ahi, Nbytes
*
*  Uses 2 bytes of stack
*
READ_MEM
*
*  Set map
*;      LDAA    0,X
*;      STAA    MAPIMG
*;      STAA    MAPREG
*
*  Get address
        LDAA    2,X                     MSB OF ADDRESS IN A
        LDAB    1,X                     LSB OF ADDRESS IN B
        STD	MEMPTR
*
*  Prepare return buffer: FN (unchanged), LEN, DATA
        LDAB    3,X                     NUMBER OF BYTES TO RETURN
        STAB    COMBUF+1                RETURN LENGTH = REQUESTED DATA
        BEQ     GLP90                   JIF NO BYTES TO GET
*
*  Read the requested bytes from local memory
        LDX     #COMBUF+2
        STX	BUFPTR
GLP     JSR     READMEM                 GET BYTE FROM MEMORY
	JSR	INCMEM
        JSR     WRITEBUF                STORE TO RETURN BUFFER
        JSR	INCBUF
        DECB
        BNE     GLP
*
*  Compute checksum on buffer, and send to master, then return
GLP90   JMP     SEND

*===========================================================================
*
*  Write Memory:  FN, len, page, Alo, Ahi, (len-3 bytes of Data)
*
*  Uses 2 bytes of stack
*
WRITE_MEM
*
*  Set map
*;      LDAA    0,X
*;      STAA    MAPIMG
*;      STAA    MAPREG
*
*  Get address
        LDAA    2,X                     MSB OF ADDRESS IN A
        LDAB    1,X                     LSB OF ADDRESS IN B
        STD	MEMPTR
*
*  Prepare return buffer: FN (unchanged), LEN, DATA
        LDAB    COMBUF+1                NUMBER OF BYTES TO WRITE
        SUBB    #3                      MINUS PAGE AND ADDRESS
        BEQ     WLP50                   JIF NO BYTES TO PUT
*
*  Write the specified bytes to local memory
        LDX     #COMBUF+5
        STX     BUFPTR
WLP     JSR     READBUF                 GET BYTE FROM BUFFER
	JSR	INCBUF
        JSR     WRITEMEM		WRITE TO MEMORY
        JSR	INCMEM
        DECB
        BNE     WLP
*
*  Compare to see if the write worked
        LDAA    2,X                     MSB OF ADDRESS IN A
        LDAB    1,X                     LSB OF ADDRESS IN B
        STD	MEMPTR
        LDAB    COMBUF+1                NUMBER OF BYTES TO WRITE
        SUBB    #3                      MINUS PAGE AND ADDRESS
        LDX     #COMBUF+5
        STX	BUFPTR
WLP20   PSHB
	JSR     READMEM                 GET BYTE JUST WRITTEN
	TAB
	JSR	READBUF
        CBA
        BNE     WLP80                   BR IF WRITE FAILED
	JSR	INCMEM
	JSR	INCBUF
        PULB
        DECB
        BNE     WLP20
*
*  Write succeeded:  return status = 0
WLP50   LDAA    #0                      RETURN STATUS = 0
        BRA     WLP90
*
*  Write failed:  return status = 1
WLP80   PULB
	LDAA    #1
*
*  Return OK status
WLP90   JMP     SEND_STATUS

*===========================================================================
*
*  Read registers:  FN, len=0
*
*  Entry with A=function code, B=data size, X=COMBUF+2
*
*  Uses 2 bytes of stack
*
READ_REGS
*
*  Enter here from SWI after "RUN" and "STEP" to return task registers
*  CAUTION:  in this case, assume no registers!
RETURN_REGS
        LDX     #TASK_REGS              POINTER TO REGISTERS
        STX	MEMPTR
        LDAB    #TASK_REG_SZ            NUMBER OF BYTES
        STAB    COMBUF+1                SAVE RETURN DATA LENGTH
*
*  Copy the registers
        LDX     #COMBUF+2               POINTER TO RETURN BUFFER
        STX	BUFPTR
GRLP    JSR	READMEM                 GET BYTE TO A
        JSR	INCMEM
        JSR	WRITEBUF                STORE TO RETURN BUFFER
        JSR	INCBUF
        DECB
        BNE     GRLP
*
*  Compute checksum on buffer, and send to master, then return
        JMP     SEND

*===========================================================================
*
*  Write registers:  FN, len, (register image)
*
*  Entry with A=function code, B=data size, X=COMBUF+2
*  Uses 2 bytes of stack
*
WRITE_REGS
	STX	BUFPTR			POINTER TO COMBUF+2
        LDAB    COMBUF+1                NUMBER OF BYTES
        BEQ     WRR80                   JIF NO REGISTERS
*
*  Copy the registers
        LDX     #TASK_REGS              POINTER TO REGISTERS
        STX	MEMPTR
WRRLP   JSR	READBUF                 GET BYTE FROM COMM BUFFER
	JSR	INCBUF
        JSR	WRITEMEM                STORE TO REGISTER RAM
        JSR	INCMEM
        DECB
        BNE     WRRLP
*
*  Return OK status
WRR80   LDAA    #0
        JMP     SEND_STATUS

*===========================================================================
*
*  Run Target:  FN, len
*
RUN_TARGET
*
*  Restore user's map
*;      LDAA    REG_PAGE                USER'S PAGE
*;      STAA    MAPIMG                  SET IMAGE
*;      STAA    MAPREG                  SET MAPPING REGISTER
*
*  Switch to user stack
        LDAB    REG_SP                  BACK TO USER STACK
        LDAA    REG_SP+1
        STD	BUFPTR
        LDS	BUFPTR
*
*  Restore registers
        LDAA    REG_PC                  SAVE LS USER PC FOR RTI
        PSHA
        LDAA    REG_PC+1                SAVE MS USER PC FOR RTI
        PSHA
*
        LDAA    REG_X
        PSHA
        LDAA    REG_X+1
        PSHA
*
        LDAA    REG_A
        PSHA
        LDAA    REG_B
        PSHA
*
        LDAA    REG_CC                  SAVE USER CONDITION CODES FOR RTI
        PSHA
*
*  Return to user
        RTI
*
*===========================================================================
*
*  Set target byte(s):  FN, len { (page, alow, ahigh, data), (...)... }
*
*  Entry with A=function code, B=data size, X=COMBUF+2
*  Return has FN, len, (data from memory locations)
*
*  If error in insert (memory not writable), abort to return short data
*
*  This function is used primarily to set and clear breakpoints
*
*  Uses 2 bytes of stack
*
SET_BYTES
        STX	BUFPTR			PTR TO (PAGE,ADDR,BYTE) TRIPLES
	STX	TEMPWORD		PTR TO RETURN BUFFER
        LDAA    #0
        STAA    COMBUF+1		SET RETURN COUNT AS ZERO
        LSRB
        LSRB                            LEN/4 = NUMBER OF BYTES TO SET
        BEQ     SB99                    JIF NO BYTES (COMBUF+1 = 0)
*
*  Loop on inserting bytes
SB10    PSHB                            SAVE LOOP COUNTER
	LDX	BUFPTR
*
*  Set map
*;      LDAA    0,X
*;      STAA    MAPIMG
*;      STAA    MAPREG
*
*  Get address
        LDAA    2,X                     MSB OF ADDRESS IN A
        LDAB    1,X                     LSB OF ADDRESS IN B
	STD	MEMPTR
	INX				BYPASS PAGE AND ADDRESS...
	INX				... POINT AT BYTE TO WRITE
	INX
	STX	BUFPTR
*
*  Read current data at byte location
        JSR	READMEM
        PSHB				STACK HAS: BYTE,COUNT,RETURN
*
*  Insert new data at byte location
	JSR	READBUF
        JSR	INCBUF
        TAB
        JSR	WRITEMEM                WRITE TARGET MEMORY
*
*  Verify write
	JSR	READMEM                 READ TARGET MEMORY
	CBA				COMPARE TO BYTE WRITTEN
        PULA				RESTORE ORIGINAL BYTE
        PULB                            RESTORE LOOP COUNTER (CC'S INTACT)
        BNE     SB90                    BR IF INSERT FAILED: ABORT
*
*  Save target byte in return buffer
	LDX	TEMPWORD
        STAA    0,X
        INX                             ADVANCE TO NEXT RETURN BYTE
        STX	TEMPWORD
        INC     COMBUF+1                COUNT ONE RETURN BYTE
*
*  Loop for next byte
        DECB
        BNE     SB10                    LOOP FOR ALL BYTES
*
*  Return buffer with data from byte locations
SB90
*
*  Compute checksum on buffer, and send to master, then return
SB99    JMP     SEND

*===========================================================================
*
*  Input from port:  FN, len, PortAddressLo, PAhi (=0)
*
*  While the 6303 has no input or output instructions, we retain these
*  to allow write-without-verify
*
IN_PORT
*
*  Get port address
        LDAA    1,X                     MSB OF ADDRESS IN A
        LDAB    0,X                     LSB OF ADDRESS IN B
        STD	MEMPTR
*
*  Read the requested byte from local memory
        JSR	READMEM
*
*  Return byte read as "status"
        JMP     SEND_STATUS

*===========================================================================
*
*  Output to port:  FN, len, PortAddressLo, PAhi (=0), data
*
*  Entry with A=function code, B=data size, X=COMBUF+2
*
OUT_PORT
*
*  Get port address
        LDAA    1,X                     MSB OF ADDRESS IN A
        LDAB    0,X                     LSB OF ADDRESS IN B
        STD	MEMPTR
*
*  Get data
        LDAA    2,X
*
*  Write value to port
        JSR	WRITEMEM
*
*  Do not read port to verify (some I/O devices don't like it)
*
*  Return status of OK
        LDAA    #0
        JMP     SEND_STATUS

*===========================================================================
*  Build status return with value from "A"
*
SEND_STATUS
        STAA    COMBUF+2                SET STATUS
        LDAA    #1
        STAA    COMBUF+1                SET LENGTH
        BRA     SEND

*===========================================================================
*  Append checksum to COMBUF and send to master
*
SEND    JSR     CHECKSUM                GET A=CHECKSUM, X->checksum location
        NEGA
        STAA    0,X                     STORE NEGATIVE OF CHECKSUM
*
*  Send buffer to master
        LDX     #COMBUF                 POINTER TO DATA
        LDAB    1,X                     LENGTH OF DATA
        ADDB    #3                      PLUS FUNCTION, LENGTH, CHECKSUM
SND10   LDAA    0,X
        JSR     PUTCHAR                 SEND A BYTE
        INX
        DECB
        BNE     SND10
*
        JMP     MAIN                    BACK TO MAIN LOOP

*===========================================================================
*  Compute checksum on COMBUF.  COMBUF+1 has length of data,
*  Also include function byte and length byte
*
*  Returns:
*       A = checksum
*       X = pointer to next byte in buffer (checksum location)
*       B is scratched
*
*  Uses 2 bytes of stack including return address
*
CHECKSUM
        LDX     #COMBUF                 pointer to buffer
        LDAB    1,X                     length of message
        ADDB    #2                      plus function, length
        LDAA    #0                      init checksum to 0
CHK10   ADDA    0,X
        INX
        DECB
        BNE     CHK10                   loop for all
        RTS                             return with checksum in A

***********************************************************************
*
*  Interrupt handlers to catch unused interrupts and traps
*  Registers are stacked.  Jump through RAM vector with type in A
*
*  This will affect only interrupt routines looking for register values!
*
*  Our default handler uses the code in "A" as the processor state to be
*  passed back to the host.
*
TRAP_ENT        LDAA    #9
                JMP     INT_ENTRY
SCI_ENT         LDAA    #8
                JMP     INT_ENTRY
TOF_ENT         LDAA    #7
                JMP     INT_ENTRY
OCF_ENT         LDAA    #6
                JMP     INT_ENTRY
ICF_ENT         LDAA    #5
                JMP     INT_ENTRY
IRQ_ENT         LDAA    #4
                JMP     INT_ENTRY
NMI_ENT         LDAA    #2
                JMP     INT_ENTRY
*
*  Non-RAM vectored
SWI_ENT         LDAA    #1              1FFC
                JMP     INT_ENTRY
*
*  INTERRUPT VECTORS
        ORG     HARD_VECT

        FDB     TRAP_ENT              1FEE TRAP (6303)
        FDB     SCI_ENT               1FF0 SCI
        FDB     TOF_ENT               1FF2 TOF
        FDB     OCF_ENT               1FF4 OCF
        FDB     ICF_ENT               1FF6 ICF
        FDB     IRQ_ENT               1FF8 IRQ/IS3
        FDB     SWI_ENT               1FFA SWI
        FDB     NMI_ENT               1FFC NMI
        FDB     RESET                 1FFE reset
*
        END     RESET
